﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections;
using Infragistics.Controls.Grids;

namespace TestCustomColumn
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            Grid.RowDefinitions.Add(new RowDefinition() {Height = GridLength.Auto });
            Grid.RowDefinitions.Add(new RowDefinition());
            MainGrid.AutoGenerateColumns = false;
            MainGrid.ExpansionIndicatorSettings.Visibility = System.Windows.Visibility.Collapsed;
            var level3 = GenerateLayout("Children");
            var level2 = GenerateLayout("Children");
            var level1 = GenerateLayout("XamWebGrid_Root_XamWebGrid");
            level2.Columns.Add(level3);
            level1.Columns.Add(level2);

            MainGrid.ColumnLayouts.Add(level1);
            MainGrid.ItemsSource = GenerateSampleData();
        }

        private Infragistics.Controls.Grids.ColumnLayout GenerateLayout(string key)
        {
            var layout = new ColumnLayout() { Key = key };
            if (key.Equals("Children"))
            {
                layout.Indentation = 0;
            }

            for (int i = 0; i < 3; i++)
            {
                layout.Columns.Add(new ExpandColumn()
                {
                    Key = "[key" + i + "]"
                });
            }
            for (int i = 0; i < 5; i++)
            {
                layout.Columns.Add(new TextColumn()
                {
                    Key = "[datakey" + i + "]"
                });
            }
            return layout;
        }

        private System.Collections.IEnumerable GenerateSampleData()
        {
            var list = new List<DataItem>();
            for (int i = 0; i < 10; i++)
            {
                var row = new DataItem();
                for (int j = 0; j < 5; j++)
                {
                    row["datakey"+j] = i+j;
                }
                row.Key = "key0";
                row.Value = i;

                row["key1"] = null;
                row["key2"] = null;

                for (int j = 0; j < 5; j++)
                {
                    var row2 = new DataItem();
                    for (int k = 0; k < 5; k++)
                    {
                        row2["datakey" + k] = j + k;
                        row2.Key = "key1";
                        row2.Value = j;

                        row2["key0"] = null;
                        row2["key2"] = null;
                    }
                    for (int n = 0; n < 5; n++)
                    {
                        var row3 = new DataItem();
                        for (int k = 0; k < 5; k++)
                        {
                            row3["datakey" + k] = n + k;
                            row3.Key = "key2";
                            row3.Value = n;

                            row3["key0"] = null;
                            row3["key1"] = null;
                        }
                        row2.Children.Add(row3);
                    }

                    row.Children.Add(row2);
                }
                list.Add(row);
            }
            return list;
        }
    }

    public class DataItem : IEnumerable
    {
        public Dictionary<string, object> Data { get; set; }
        public List<DataItem> Children { get; set; }

        public string Key { get; set; }

        private object _Value;
        public object Value
        {
            get
            {
                return _Value;
            }
            set
            {
                _Value = value;
                this[Key] = value;
            }
        }

        public DataItem()
        {
            Data = new Dictionary<string, object>();
            Children = new List<DataItem>();
        }

        public object this[string key]
        {
            get
            {
                return Data[key];
            }
            set
            {
                Data[key] = value;
            }
        }

        public IEnumerator GetEnumerator()
        {
            return Data.GetEnumerator();
        }

        public override string ToString()
        {
            return Key + " " + Value;
        }
    }
}
